/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-build.h"

/* Run the autogen.sh command if a project is currently open. */
void 
openldev_menu_build_auto_generate (OpenLDev *openldev)
{
  gchar *command, *message;
  gboolean result;
  GtkWidget *output;
  ProjectSettings *project;
  StatusBar *statusbar;
  EnvironmentSettings *env;

  g_object_get (openldev, "output-pane", &output, "project", &project,
                "status-bar", &statusbar, "env-settings", &env, NULL);
  
  gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
  chdir (project->location);
  
  command = g_strconcat (project->autogen, " 2>&1", NULL);
  message = g_strconcat (_("Running: "), project->autogen, NULL);
  openldev_status_bar_lock_message (statusbar, message);
  result = output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), command, env);
  
  if (result || !result)
    openldev_status_bar_unlock_message (statusbar);
}

/* If a project is open, build it. Run autogen.sh and then build again if
 * Makefiles are not found. All of the files are first saved. */
void 
openldev_menu_build_project (OpenLDev *openldev)
{
  gchar *command, *message;
  gboolean result;
  GtkWidget *output;
  ProjectSettings *project;
  StatusBar *statusbar;
  EnvironmentSettings *env;

  g_object_get (openldev, "output-pane", &output, "project", &project,
                "status-bar", &statusbar, "env-settings", &env, NULL);

  gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
  openldev_menu_file_save_all (openldev);
  chdir (project->location);
    
  command = g_strconcat (project->build, " 2>&1", NULL);
  message = g_strconcat (_("Running: "), project->build, NULL);
  openldev_status_bar_lock_message (statusbar, message);

  if (!output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), command, env))
  {
    openldev_status_bar_unlock_message (statusbar);
    openldev_menu_build_auto_generate (openldev);
    openldev_status_bar_lock_message (statusbar, message);
    result = output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), command, env);
      
    if (result || !result)
      openldev_status_bar_unlock_message (statusbar);
  }
  else
    openldev_status_bar_unlock_message (statusbar);
}

/* Clean all of the sources, build files, object files and those files
 * created by configure. */
void 
openldev_menu_build_clean (OpenLDev *openldev)
{
  gchar *command, *message;
  gboolean result;
  GtkWidget *output;
  ProjectSettings *project;
  StatusBar *statusbar;
  EnvironmentSettings *env;

  g_object_get (openldev, "output-pane", &output, "project", &project,
                "status-bar", &statusbar, "env-settings", &env, NULL);

  gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
  chdir (project->location);
    
  command = g_strconcat (project->clean, " 2>&1", NULL);
  message = g_strconcat (_("Running: "), project->clean, NULL);
  openldev_status_bar_lock_message (statusbar, message);
  result = output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), command, env);
    
  if (result || !result)
    openldev_status_bar_unlock_message (statusbar);
}

/* Compile current file if a file and a project is open. The function then
 * creates the compile string from the project language & runs the command. */
void 
openldev_menu_build_compile (OpenLDev *openldev)
{
  GString *command;
  gchar *message, *location, *fn;
  gboolean result;
  gint loc;
  GtkWidget *output, *files;
  ProjectSettings *project;
  StatusBar *statusbar;
  EnvironmentSettings *env;

  g_object_get (openldev, "output-pane", &output, "project", &project,
                "status-bar", &statusbar, "env-settings", &env,
                "file-browser", &files, NULL);

  gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
  command = g_string_new (project->compile);
  fn = file_manager_get_current_filename (FILE_MANAGER (files));
    
  loc = g_strstr_len (command->str, command->len, "{FILE}") - command->str;
  while (loc >= 0)
  {
    command = g_string_erase (command, loc, 6);
    command = g_string_insert (command, loc, fn);
    loc = g_strstr_len (command->str, command->len, "{FILE}") - command->str;
  }
    
  message = g_strconcat (_("Running: "), command->str, NULL);
  location = g_strconcat (project->location, "src/", NULL);
  chdir (project->location);

  command = g_string_append (command, " 2>&1");
  openldev_status_bar_lock_message (statusbar, message);
  result = output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), command->str, env);
    
  if (result || !result)
    openldev_status_bar_unlock_message (statusbar);
}

/* If a project is open, run the ./configure command in the project directory. */
void 
openldev_menu_build_configure (OpenLDev *openldev)
{
  gchar *message, *command, *conf;
  GString *configure;
  gboolean result;
  GtkWidget *output;
  ProjectSettings *project;
  StatusBar *statusbar;
  EnvironmentSettings *env;

  g_object_get (openldev, "output-pane", &output, "project", &project,
                "status-bar", &statusbar, "env-settings", &env, NULL);

  configure = g_string_new (project->configure);
  if (g_strstr_len (configure->str, strlen (configure->str), "./configure"))
    configure = g_string_erase (configure, 0, 11);
  g_strchug (configure->str);
  conf = openldev_input_dialog_run (_("Parameters:"), _("Configure Parameters"), configure->str);
    
  if (g_ascii_strcasecmp (conf, "EMPTY") != 0)
  {
    gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
    chdir (project->location);
    
    if (strlen (conf) > 0)
      command = g_strconcat ("./configure ", conf, NULL);
    else
      command ="./configure";
      
    project->configure = g_strdup (command);
    message = g_strconcat (_("Running: "), command, NULL);
    openldev_status_bar_lock_message (statusbar, message);
    result = output_build_run_command (OUTPUT_BUILD (OUTPUT (output)->build), 
                                       g_strconcat (command, " 2>&1", NULL), env);
   
    if (result || !result)
      openldev_status_bar_unlock_message (statusbar);
  }
}

/* If a project is open, run the executable in the terminal specified by the user. */
void 
openldev_menu_build_execute (OpenLDev *openldev)
{
  gchar *command;
  gint pid;
  ProjectSettings *project;

  g_object_get (openldev, "project", &project, NULL);

  command = g_strconcat ("gnome-terminal -e \"openldev_launcher ", project->location, 
                         "src/", project->name, "\"", NULL);    
  pid = openldev_fork_new_process (command);
}

/* Stop the current compilation, clean, configure, autogen or build. */
void 
openldev_menu_build_stop (OpenLDev *openldev)
{ 
  OutputManager *output;

  g_object_get (openldev, "output-pane", &output, NULL);
  output_build_stop_build (OUTPUT_BUILD (output->build)); 
}
