/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
 
#include "openldev-file.h"

#define NUM_MENUITEMS 17
gchar* menuitems[] = { "/MenuBar/File/FileSave", "/MenuBar/File/FileSaveAs", 
       "/MenuBar/File/FileSaveAll", "/MenuBar/File/FileRevert", "/MenuBar/File/FilePrint", 
       "/MenuBar/File/FilePrintPreview", "/MenuBar/Edit/EditUndo", "/MenuBar/Edit/EditRedo", 
       "/MenuBar/Edit/EditCut", "/MenuBar/Edit/EditCopy", "/MenuBar/Edit/EditPaste", 
       "/MenuBar/Edit/EditSelectAll", "/MenuBar/Edit/EditDeselect", "/ToolBar/FilePrint",
       "/ToolBar/FileSave", "/ToolBar/EditUndo", "/ToolBar/EditRedo" };

void openldev_on_delete_tab (GtkWidget*, gpointer);
void openldev_enable_project_menu_items (OpenLDev*);
void openldev_file_connect_signals (OpenLDev*);
void openldev_add_recent_file (OpenLDev*, const char*);
void openldev_upgrade_project (string&);

// Create a new file. This doesn't add the file to the project. The user must
// do that manually through the makefile editor.
void 
openldev_menu_file_new_file (OpenLDev *openldev)
{
  gchar *file;
  NewFileDialog *dialog;
  ProjectSettings *project;
  SideBar *sidebar;
  
  g_object_get (openldev, "project", &project, "sidebar", &sidebar, NULL);
  dialog = openldev_new_file_dialog_new (project->location);
  openldev_new_file_dialog_run (dialog, file);
    
  if (g_ascii_strcasecmp (file, "NULL") != 0 && strlen (file) > 0)
  {
    file_browser_refresh (FILE_BROWSER (sidebar_get_file_browser (sidebar)));
    openldev_open_file (openldev, file);     
  }
    
  gtk_widget_destroy (dialog->dialog);
  g_object_unref (dialog->xml);
  g_slice_free1 (sizeof (NewFileDialog), dialog);
}

/* This function is where you would apply the changes and destroy the assistant. */
static void 
npd_assistant_close (GtkWidget *assistant, 
                     NewProjectDialog *npd)
{
  gtk_widget_hide_all (assistant);
  
  if (npd != NULL)
  {
    gchar *directory, *fn, *type, *pf, *comm;
    GList *entries = NULL, *real_entries = NULL, *ptr;
    OpenLDev *openldev;
    OutputManager *output;
    EnvironmentSettings *env;
    GString *script;
    
    openldev = (OpenLDev*) g_object_get_data (G_OBJECT (assistant), "openldev");
    g_object_get (openldev, "output-pane", &output, "env-settings", &env, NULL);
  
    type = gtk_combo_box_get_active_text (GTK_COMBO_BOX (npd->combobox));
    directory = openldev_add_prefix ("/share/openldev/templates/");
    GDir *dir = g_dir_open (directory, 0, NULL);
    while ((fn = (gchar*) g_dir_read_name (dir)) != NULL)
      entries = g_list_prepend (entries, (gpointer) g_strdup (fn));
  
    for (ptr = entries; ptr != NULL; ptr = ptr->next)
      if (g_strrstr ((gchar*) ptr->data, ".sh") != NULL)
        real_entries = g_list_prepend (real_entries, ptr->data);

    for (ptr = real_entries; ptr != NULL; ptr = ptr->next) 
    {
      gchar buff[8], *command;
      GString *text;
      FILE *in;
      gint i;
    
      command = g_strdup_printf ("sh %s%s name", directory, (gchar*) ptr->data);
      text = g_string_new (NULL);
      in = popen (command, "r");
      while (fgets (buff, sizeof (buff), in))
      {
        i = 0;
        while (buff[i] != '\0' && i < 8)
        {
          text = g_string_append_c (text, buff[i]);
          i++;
        }
      }
      pclose (in);

      if (g_ascii_strcasecmp (text->str, type) == 0)
      {
        script = g_string_new (directory);
        script = g_string_append (script, (gchar*) ptr->data);
        break;
      }
      
      g_string_free (text, TRUE);
    }

    pf =   g_strdup_printf ("%s/%s/%s.dev", 
                            gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (npd->location)),
                            gtk_entry_get_text (GTK_ENTRY (npd->name)),
                            gtk_entry_get_text (GTK_ENTRY (npd->name)));
    comm = g_strdup_printf ("sh %s '%s/%s/' '%s' '%s' '%s' '%s' '%s'", script->str,
                            gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (npd->location)),
                            gtk_entry_get_text (GTK_ENTRY (npd->name)),
                            gtk_entry_get_text (GTK_ENTRY (npd->name)),
                            gtk_entry_get_text (GTK_ENTRY (npd->author)),
                            gtk_entry_get_text (GTK_ENTRY (npd->homepage)),
                            gtk_entry_get_text (GTK_ENTRY (npd->version)),
                            gtk_entry_get_text (GTK_ENTRY (npd->email)));
  
    gtk_notebook_set_current_page (GTK_NOTEBOOK (output), 0);
    gboolean result = output_build_run_command (OUTPUT_BUILD (output->build), 
                                                comm, env);

    if (result)
      if (openldev_open_project (openldev, pf))
        openldev_menu_build_auto_generate (openldev);
    
    g_list_foreach (entries, (GFunc) g_free, NULL);
    g_list_free (entries);
    g_list_free (real_entries);
    g_string_free (script, TRUE);
  }
}

// Create a new project by calling the new project dialog & then open the
// project that was created.
void openldev_menu_file_new_project (OpenLDev *openldev)
{
  NewProjectDialog *npd = openldev_new_project_dialog_new ((gpointer) openldev);
  
  g_signal_connect (G_OBJECT (npd->assistant), "cancel",
                    G_CALLBACK (npd_assistant_close), NULL);
  g_signal_connect (G_OBJECT (npd->assistant), "close",
                    G_CALLBACK (npd_assistant_close), (gpointer) npd);
}

// Get the file that needs to be opened. This function doesn't actually open the
// file. Therefore, it is tricky and lies, but oh well.
void openldev_menu_file_open (OpenLDev *openldev)
{
  GtkWidget *files;
  // Create file selector
  GtkWidget *dialog = gtk_file_chooser_dialog_new (_("Choose a File"), GTK_WINDOW (openldev),
                      GTK_FILE_CHOOSER_ACTION_OPEN, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                      GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
  gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER (dialog), TRUE);
  
  // Set default folder
  g_object_get (openldev, "file-browser", &files, NULL);
  if (file_manager_get_file_open (FILE_MANAGER (files)))
  {
    int l;
    string temp = file_manager_get_current_filename (FILE_MANAGER (files)), file;
    
    l = temp.find ("/", 0);
    while (l != string::npos)
    {
      file += temp.substr (0, l + 1);
      temp.erase (0, l + 1);
      l = temp.find ("/", 0);
    }
    
    gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), file.c_str());
  } 
  
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .dev file
      GSList *filenames = gtk_file_chooser_get_filenames (GTK_FILE_CHOOSER (dialog));
      for (gint i=0; i < g_slist_length (filenames); i++)
        openldev_open_file (openldev, (gchar*) g_slist_nth_data (filenames, i));
    }
    default:
      gtk_widget_destroy (dialog);
      break;
  }
}

void openldev_menu_file_open_project (OpenLDev *openldev)
{
  // Create file selector
  GtkFileFilter *filter = gtk_file_filter_new ();
  gtk_file_filter_set_name (filter, _("OpenLDev Project Files"));
  gtk_file_filter_add_pattern (filter, "*.dev");
  gtk_file_filter_add_pattern (filter, "*.openldev");
  
  GtkWidget *dialog = gtk_file_chooser_dialog_new (_("Choose a Project"), 
                      GTK_WINDOW (openldev), GTK_FILE_CHOOSER_ACTION_OPEN, 
                      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                      GTK_STOCK_OPEN, GTK_RESPONSE_OK, NULL);
  gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (dialog), filter);
  
  int result = gtk_dialog_run (GTK_DIALOG (dialog));
  switch (result)
  {
    // If OK is pressed
    case (GTK_RESPONSE_OK):
    {
      // Check if file is a .dev file
      string filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
      gtk_widget_destroy (dialog);
      gboolean result_val;
      
      if (filename.find (".dev", 0) != string::npos ||
          filename.find (".openldev", 0) != string::npos)
        result_val = openldev_open_project (openldev, (gchar*) filename.c_str());
      break;
    }
    default:
      gtk_widget_destroy (dialog);
      break;
  }
}

// Open the specified file if it exists. This function receives the filename that
// it is supposed to open. If a project is open, the file is added as the
// last opened file. The file is then opened, syntax highlighting is applied, 
// line numbering is calculated and displayed, signals are connected to the
// keystroke action and final initialization is applied
gboolean openldev_open_file (OpenLDev *openldev,
                             gchar *file)
{
  string filename = file;
  ProjectSettings *project;
  GtkWidget *files;
  EnvironmentSettings *env;
  
  g_object_get (openldev, "project", &project, "file-browser", &files,
                "env-settings", &env, NULL);
  
  if (!g_file_test (filename.c_str(), G_FILE_TEST_EXISTS))
  {
    openldev_error (_("Cannot open file. It does not exist!"));
    if (project->name == NULL)
      openldev_project_settings_remove_lastfile (project, (gchar*) filename.c_str());
    return FALSE;
  }
  else
  {
    // Add a new tab to the notebookmanager
    if (file_manager_add_tab (FILE_MANAGER (files), filename.c_str(), env))
    {
      // Set the file as a last opened file
      if (project->name != NULL)
        openldev_project_settings_add_lastfile (project, (gchar*) filename.c_str());
  
      // Open file & read in text
      GtkSourceBuffer *buffer = file_manager_get_current_buffer (FILE_MANAGER (files));
      
      // Connect keystroke function & show/hide line numbering
      file_manager_set_properties (FILE_MANAGER (files), FALSE, env);
      openldev_file_connect_signals (openldev);
      
      // Set the font for the new file
      FileTab *tab = FILE_TAB (FILE_MANAGER (files)->tabs[FILE_MANAGER (files)->tab_count-1]);
      string font = g_strconcat (env->font_face, " ", env->font_size, NULL);
      PangoFontDescription *fd = pango_font_description_from_string (font.c_str());
      PangoTabArray *tabs = file_manager_get_tab_array (fd, env, GTK_WIDGET (tab));
      gtk_text_view_set_tabs (GTK_TEXT_VIEW (tab), tabs);
      file_manager_modify_fonts (FILE_MANAGER (files), fd);
      openldev_manage_menu_items (openldev);
      openldev_add_recent_file (openldev, filename.c_str());
    }
  }
  
  return TRUE;
}

void openldev_add_recent_file (OpenLDev *openldev,
                               const char *filename)
{
  EnvironmentSettings *env;
  MenuHandle *menus;

  g_object_get (openldev, "env-settings", &env, "menus", &menus, NULL);

  // Add a recent file & then repopulate them
  openldev_environment_add_recent_file (env, (gchar*) filename);
  
  for (int i=0; i < 5; i++)
  {
    string recent = "/MenuBar/File/FileRecent";
    gchar buff[2];
    sprintf (buff, "%d", i+1);
    recent += buff;
  
    if (g_slist_length (env->recent_files) <= i)
      gtk_widget_hide (gtk_ui_manager_get_widget (menus->uimanager, recent.c_str()));
    else
    {
      string name = string (buff) + ". " + string (openldev_environment_get_recent_short (env, i));
      string tooltip = string ("Open \"") + (gchar*) g_slist_nth_data 
                       (env->recent_files, i) + string ("\"");
      gtk_label_set_text (GTK_LABEL (gtk_bin_get_child (GTK_BIN (gtk_ui_manager_get_widget 
                          (menus->uimanager, recent.c_str())))), name.c_str());
      gtk_widget_show (gtk_ui_manager_get_widget (menus->uimanager, recent.c_str()));
      gtk_tooltips_set_tip (menus->tooltips, gtk_ui_manager_get_widget 
                            (menus->uimanager, recent.c_str()), tooltip.c_str(), tooltip.c_str());
    }
  }
}

// Connect the signals for the newly opened file
void openldev_file_connect_signals (OpenLDev *openldev)
{
  FileManager *files;
  
  g_object_get (openldev, "file-browser", &files, NULL);
  g_signal_connect (G_OBJECT (file_manager_get_current_view (files)), 
                    "key-press-event", G_CALLBACK (openldev_on_textview_keypress), 
                    (gpointer) openldev);
  g_signal_connect (G_OBJECT (file_manager_get_current_view (files)), 
                    "event", G_CALLBACK (openldev_on_textview_event), (gpointer) openldev);
  g_signal_connect (G_OBJECT (FILE_TAB (files->tabs[files->tab_count-1])->button), "clicked",
                    G_CALLBACK (openldev_on_delete_tab), (gpointer) openldev);
}

// The project specified is opened. This function receives the filename of the
// project file. If the project was loaded, the last file opened is reloaded,
// the filelist is populated and the last project open is set to this project
gboolean openldev_open_project (OpenLDev *openldev,
                                gchar *file)
{
  string filename = file;
  GSList *list = NULL;
  FileManager *files;
  ProjectSettings *project;
  EnvironmentSettings *env;
  
  g_object_get (openldev, "project", &project, "file-browser", &files,
                "env-settings", &env, NULL);
  
  if (filename.find (".openldev") != string::npos)
  {
    openldev_error (_("<span weight='bold'>OpenLDev Message:</span>\n\nThis project file is no longer compatible with OpenLDev! You have two options.\n\n1. You can create a new project and import your sources into that project.\n\n2. You can use OpenLDev Switch available at http://mannequin.invigorated.org to upgrade your project."));
    return FALSE;
  }

  file_manager_erase_all_tabs (files);
  gboolean success = openldev_project_settings_load_project_file (project, 
                                                                  (gchar*) filename.c_str());
 
  // If the project was opened
  if (success)
  {
    // Open the last files if they exist
    for (list = project->lastfiles; list != NULL; list = list->next)
      openldev_open_file (openldev, (gchar*) list->data);

    // Make environment ready for project
    openldev_populate_treeview (openldev);
    openldev_environment_set_last_project (env, (gchar*) filename.c_str());
    
    // Enable all of the project menu items because a project is now open
    openldev_enable_project_menu_items (openldev);
    g_signal_emit_by_name (G_OBJECT (openldev), "project-open", 0);
    return TRUE;
  }
  
  return FALSE;
}

// Enable all of the project menu items because a project is now open
void openldev_enable_project_menu_items (OpenLDev *openldev)
{
  MenuHandle *menus;
  
  g_object_get (openldev, "menus", &menus, NULL);

  menu_handle_set_sensitive (menus->uimanager, "/MenuBar/File/FileClose", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/MenuBar/Project", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/MenuBar/Build", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/ToolBar/BuildCompileToolbar", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/ToolBar/BuildBuildToolbar", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/ToolBar/BuildExecute", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/ToolBar/BuildClean", TRUE);
  menu_handle_set_sensitive (menus->uimanager, "/ToolBar/BuildStop", TRUE);
}

// Save the current file & mark the file as unmodified. If the file does not
// have a name, then save as is called
void openldev_menu_file_save (OpenLDev *openldev)
{
  FileManager *files;
  
  g_object_get (openldev, "file-browser", &files, NULL);

  if (file_manager_get_file_open (files))
  {
    int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (files));
    
    if (IS_FILE_TAB (FILE_TAB (files->tabs[current])))
    {
      string current_fn = file_manager_get_current_filename (files);
      if (current_fn.empty() || current_fn == "EMPTY")
        openldev_menu_file_save_as (openldev);
  
      // Save the file & set as unmodified
      GtkSourceBuffer *buffer = file_manager_get_current_buffer (files);
      openldev_write_file (file_manager_get_current_filename (files),
                           openldev_file_get_text_in_buffer (buffer));

      file_manager_mark_current_tab_modified (files, FALSE, current);
    }
  }
}

// Save the current file by a different name. The FileChooserDialog allows the
// user to choose the location and filename. Note: This function does not actually
// save the file, but rather just chooses a new name for it & calls on_file_save
void openldev_menu_file_save_as (OpenLDev *openldev)
{
  FileManager *files;
  ProjectSettings *project;
  
  g_object_get (openldev, "file-browser", &files, "project", &project, NULL);

  if (file_manager_get_file_open (files))
  {
    int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (files));
    
    if (IS_FILE_TAB (FILE_TAB (files->tabs[current])))
    {
      GtkWidget *dialog;
      dialog = gtk_file_chooser_dialog_new (_("Save file as"), 
                                            GTK_WINDOW (openldev),
                                            GTK_FILE_CHOOSER_ACTION_SAVE,
                                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                            GTK_STOCK_SAVE, GTK_RESPONSE_OK,
                                            NULL);
      
      string temp = file_manager_get_current_filename (files), file;
      int l = temp.find ("/", 0);
      while (l != string::npos)
      {
        file += temp.substr (0, l + 1);
        temp.erase (0, l + 1);
        l = temp.find ("/", 0);
      }

      // Add the filename to the text entry in Save As dialog
      gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), file.c_str());
      gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (dialog),
          file_manager_get_current_filename_short (files).c_str());
  
      // Open fileselection dialog to get file location
      int result = gtk_dialog_run (GTK_DIALOG (dialog));
      switch (result)
      {
        // If OK is pressed
        case (GTK_RESPONSE_OK):
        {
          if (project->name != NULL)
          {
            openldev_project_settings_remove_lastfile (project, 
                     file_manager_get_current_filename (files));
            openldev_project_settings_add_lastfile (project, 
                     gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
          }
        
          file_manager_set_filename (files,
               gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog)));
          openldev_menu_file_save (openldev);
          break;
        }
        default:
          break;
      }
    
      gtk_widget_destroy (dialog);
    }
  }
}

// Call the notebookmanager function to save all of the files that are
// currently open
void openldev_menu_file_save_all (OpenLDev *openldev)
{
  FileManager *files;
  
  g_object_get (openldev, "file-browser", &files, NULL);

  for (int i=0; i < gtk_notebook_get_n_pages (GTK_NOTEBOOK (files)); i++)
  {
    // Save the file & set as unmodified
    if (IS_FILE_TAB (FILE_TAB (files->tabs[i])))
    {
      if (FILE_TAB (files->tabs[i])->modified)
      {
        GtkTextBuffer *buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (files->tabs[i]));
        openldev_write_file (FILE_TAB (files->tabs[i])->filename,
                             openldev_file_get_text_in_buffer (GTK_SOURCE_BUFFER (buffer)));

        file_manager_mark_current_tab_modified (files, FALSE, i);
      }
    }
  }
}

// If the user chooses to allow it, revert to the saved version of a file. Then
// the syntax highlighting, action manager and line numbering are refreshed
void openldev_menu_file_revert (OpenLDev *openldev)
{
  FileManager *files;
  
  g_object_get (openldev, "file-browser", &files, NULL);
  int current = gtk_notebook_get_current_page (GTK_NOTEBOOK (files));
    
  if (IS_FILE_TAB (FILE_TAB (files->tabs[current])))
    if (file_manager_get_file_open (files))
      if (openldev_question (_("Are you sure you want to reload this file? Any unsaved changes will be lost.")))
        file_manager_revert_to_saved (files);
}

void openldev_on_delete_tab (GtkWidget *widget,
                             gpointer data)
{
  OpenLDev *openldev = (OpenLDev*) data;
  FileManager *files;
  ProjectSettings *project;
  
  g_object_get (openldev, "file-browser", &files, "project", &project, NULL);

  for (int i=0; i < files->tab_count; i++)
    if (FILE_TAB (files->tabs[i])->button == widget)
      openldev_project_settings_remove_lastfile (project, FILE_TAB (files->tabs[i])->filename);

  file_manager_on_delete_tab (widget, (gpointer) files);
  openldev_manage_menu_items (openldev);
}

void openldev_manage_menu_items (OpenLDev *openldev)
{
  GtkWidget *item;
  FileManager *files;
  MenuHandle *menus;
  gboolean show;
  
  g_object_get (openldev, "file-browser", &files, "menus", &menus, NULL);
  show = file_manager_get_file_open (files);
  item = gtk_ui_manager_get_widget (menus->uimanager, menuitems[0]);
  
  if (!(show == GTK_WIDGET_SENSITIVE (item)))
  {
    for (unsigned int i=0; i < NUM_MENUITEMS; i++)
    {
      item = gtk_ui_manager_get_widget (menus->uimanager, menuitems[i]);
      gtk_widget_set_sensitive (item, show);
    }
  }
}
