/***************************************************************************
 *   Copyright (C) 2004-2005 by Andrew Krause                              *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef OPENLDEV_MENU_CALLBACKS_H
#define OPENLDEV_MENU_CALLBACKS_H

#include <gtk/gtk.h>
#include "openldev-gettext.h"

typedef struct _OpenLDev  OpenLDev;

// Popup menu action handlers
void openldev_on_find_and_replace (GtkWidget*, OpenLDev*);
void openldev_refresh_file_browser (GtkMenuItem*, OpenLDev*);
void openldev_open_file_browser (GtkMenuItem*, OpenLDev*);
void openldev_clear_build_output (GtkMenuItem*, OpenLDev*);
void openldev_clear_search_output (GtkMenuItem*, OpenLDev*);
void openldev_copy_output (GtkMenuItem*, OpenLDev*);
void openldev_jump_to_build_output (GtkMenuItem*, OpenLDev*);
void openldev_jump_to_search_output (GtkMenuItem*, OpenLDev*);

// Menu handler functions
void openldev_file_close_project (GtkMenuItem*, OpenLDev*);
void openldev_file_import_project(GtkMenuItem*, OpenLDev*);
void openldev_file_new_file      (GtkMenuItem*, OpenLDev*);
void openldev_file_new_project   (GtkMenuItem*, OpenLDev*);
void openldev_file_open          (GtkMenuItem*, OpenLDev*);
void openldev_file_open_project  (GtkMenuItem*, OpenLDev*);
void openldev_file_print         (GtkMenuItem*, OpenLDev*);
void openldev_file_print_preview (GtkMenuItem*, OpenLDev*);
void openldev_file_revert        (GtkMenuItem*, OpenLDev*);
void openldev_file_save          (GtkMenuItem*, OpenLDev*);
void openldev_file_save_all      (GtkMenuItem*, OpenLDev*);
void openldev_file_save_as       (GtkMenuItem*, OpenLDev*);
void openldev_file_quit          (GtkMenuItem*, OpenLDev*);

void openldev_edit_copy        (GtkMenuItem*, OpenLDev*);
void openldev_edit_cut         (GtkMenuItem*, OpenLDev*);
void openldev_edit_deselect    (GtkMenuItem*, OpenLDev*);
void openldev_edit_paste       (GtkMenuItem*, OpenLDev*);
void openldev_edit_preferences (GtkMenuItem*, OpenLDev*);
void openldev_edit_redo        (GtkMenuItem*, OpenLDev*);
void openldev_edit_select_all  (GtkMenuItem*, OpenLDev*);
void openldev_edit_undo        (GtkMenuItem*, OpenLDev*);
void openldev_edit_find        (GtkMenuItem*, OpenLDev*);
void openldev_edit_find_next   (GtkMenuItem*, OpenLDev*);
void openldev_edit_goto_line   (GtkMenuItem*, OpenLDev*);
void openldev_edit_indent      (GtkMenuItem*, OpenLDev*);
void openldev_edit_unindent    (GtkMenuItem*, OpenLDev*);

void openldev_project_info        (GtkMenuItem*, OpenLDev*);
void openldev_project_preferences (GtkMenuItem*, OpenLDev*);

void openldev_build_auto_generate (GtkMenuItem*, OpenLDev*);
void openldev_build_clean         (GtkMenuItem*, OpenLDev*);
void openldev_build_compile       (GtkMenuItem*, OpenLDev*);
void openldev_build_configure     (GtkMenuItem*, OpenLDev*);
void openldev_build_execute       (GtkMenuItem*, OpenLDev*);
void openldev_build_project       (GtkMenuItem*, OpenLDev*);
void openldev_build_stop          (GtkMenuItem*, OpenLDev*);

void openldev_view_browser        (GtkMenuItem*, OpenLDev*);
void openldev_view_line_numbering (GtkMenuItem*, OpenLDev*);
void openldev_view_output         (GtkMenuItem*, OpenLDev*);
void openldev_view_statusbar      (GtkMenuItem*, OpenLDev*);
void openldev_view_toolbar        (GtkMenuItem*, OpenLDev*);

void openldev_help_about         (GtkMenuItem*, OpenLDev*);
void openldev_help_documentation (GtkMenuItem*, OpenLDev*);
void openldev_help_report_bug    (GtkMenuItem*, OpenLDev*);

#define NUM_ENTRIES 65
static GtkActionEntry entries[] = {
  { "File", NULL, N_("_File") },
  { "Edit", NULL, N_("_Edit") },
  { "View", NULL, N_("_View") },
  { "Search", NULL, N_("_Search") },
  { "Project", NULL, N_("_Project") },
  { "Build", NULL, N_("_Build") },
  { "Help", NULL, N_("_Help") },
  
  { "FileNewFile", GTK_STOCK_NEW, N_("_New File"), "<control>n", 
    N_("Create a new file"), G_CALLBACK (openldev_file_new_file) },
  { "FileNewFileToolbar", GTK_STOCK_NEW, N_(" New File"), NULL, 
    N_("Create a new file"), G_CALLBACK (openldev_file_new_file) },
  { "FileNewProject", "NEWPROJECT", N_("New Project"), "<control><shift>n", 
    N_("Create a new project"), G_CALLBACK (openldev_file_new_project) },
  { "FileOpen", GTK_STOCK_OPEN, N_("_Open"), "<control>o", 
    N_("Open an existing file"), G_CALLBACK (openldev_file_open) },
  { "FileOpenProject", GTK_STOCK_OPEN, N_("Open Pro_ject"), "<control><shift>o", 
    N_("Open an existing project"), G_CALLBACK (openldev_file_open_project) },
  { "FileImportProject", GTK_STOCK_CONVERT, N_("_Import Project"), NULL, 
    N_("Import a new OpenLDev project"), G_CALLBACK (openldev_file_import_project) },
  { "FileOpenToolbar", GTK_STOCK_OPEN, N_(" Open"), NULL, 
    N_("Open an existing file"), G_CALLBACK (openldev_file_open) },
  { "FileSave", GTK_STOCK_SAVE, N_("_Save"), "<control>s", 
    N_("Save current file"), G_CALLBACK (openldev_file_save) },
  { "FileSaveAs", GTK_STOCK_SAVE_AS, N_("Save _As"), "<control><shift>s", 
    N_("Save file by new name"), G_CALLBACK (openldev_file_save_as) },
  { "FileSaveAll", NULL, N_("Save All"), "<control><shift>l", 
    N_("Save all open files"), G_CALLBACK (openldev_file_save_all) },
  { "FileRevert", GTK_STOCK_REVERT_TO_SAVED, N_("_Reload File"), NULL, 
    N_("Revert to saved file"), G_CALLBACK (openldev_file_revert) },
  { "FilePrint", GTK_STOCK_PRINT, N_("_Print"), "<control>p", 
    N_("Print the current file"), G_CALLBACK (openldev_file_print) },
  { "FilePrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Preview"), "<control><shift>p", 
    N_("Preview the current file"), G_CALLBACK (openldev_file_print_preview) },
  { "FileClose", GTK_STOCK_CLOSE, N_("_Close Project"), "<control><shift>w", 
    N_("Close current project"), G_CALLBACK (openldev_file_close_project) },
  { "FileQuit", GTK_STOCK_QUIT, N_("_Quit"), "<control>q", 
    N_("Quit the application"), G_CALLBACK (openldev_file_quit) },
  { "FileRecent1", NULL, "", NULL, "", NULL },
  { "FileRecent2", NULL, "", NULL, "", NULL },
  { "FileRecent3", NULL, "", NULL, "", NULL },
  { "FileRecent4", NULL, "", NULL, "", NULL },
  { "FileRecent5", NULL, "", NULL, "", NULL },

  { "EditUndo", GTK_STOCK_UNDO, N_("_Undo"), "<control>z", 
    N_("Undo last action"), G_CALLBACK (openldev_edit_undo) },
  { "EditRedo", GTK_STOCK_REDO, N_("_Redo"), "<control><shift>z", 
    N_("Redo last undone action"), G_CALLBACK (openldev_edit_redo) },
  { "EditCut", GTK_STOCK_CUT, N_("C_ut"), "<control>x", 
    N_("Cut selected text"), G_CALLBACK (openldev_edit_cut) },
  { "EditCopy", GTK_STOCK_COPY, N_("_Copy"), "<control>c", 
    N_("Copy selected text"), G_CALLBACK (openldev_edit_copy) },
  { "EditPaste", GTK_STOCK_PASTE, N_("_Paste"), "<control>v", 
    N_("Paste text from clipboard"), G_CALLBACK (openldev_edit_paste) },
  { "EditSelectAll", NULL, N_("Select _All"), "<control>a", 
    N_("Select all text"), G_CALLBACK (openldev_edit_select_all) },
  { "EditDeselect", NULL, N_("_Deselect"), NULL, 
    N_("Deselect selected text"), G_CALLBACK (openldev_edit_deselect) },
  { "EditFind", GTK_STOCK_FIND, N_("_Find"), "<control>f", 
    N_("Open the Find toolbar"), G_CALLBACK (openldev_edit_find) },
  { "EditFindNext", GTK_STOCK_FIND, N_("Find _Next"), "<control>g", 
    N_("Find next"), G_CALLBACK (openldev_edit_find_next) },
  { "EditReplace", GTK_STOCK_FIND_AND_REPLACE, N_("Replace"), "<control>r", 
    N_("Find and replace"), G_CALLBACK (openldev_on_find_and_replace) },
  { "EditGotoLine", GTK_STOCK_JUMP_TO, N_("_Goto Line"), "<control>i", 
    N_("Open Goto Line toolbar"), G_CALLBACK (openldev_edit_goto_line) },
  { "EditIndent", GTK_STOCK_INDENT, N_("Indent"), "<control>t", 
    N_("Indent the current line"), G_CALLBACK (openldev_edit_indent) },
  { "EditUnIndent", GTK_STOCK_UNINDENT, N_("Unindent"), "<control><shift>t", 
    N_("Unindent the current line"), G_CALLBACK (openldev_edit_unindent) },
  { "EditSettings", GTK_STOCK_PREFERENCES, N_("Preferences"), NULL, 
    N_("Edit Environment settings"), G_CALLBACK (openldev_edit_preferences) },
  
  { "ViewFiles", GTK_STOCK_JUSTIFY_LEFT, N_("_Project Files"), NULL, 
    N_("Show or hide project files"), G_CALLBACK (openldev_view_browser) },
  { "ViewOutput", GTK_STOCK_JUSTIFY_LEFT, N_("_Build Output"), NULL, 
    N_("Show or hide build output"), G_CALLBACK (openldev_view_output) },
  
  { "ProjectConfigure", GTK_STOCK_PREFERENCES, N_("_Configure Project"), NULL, 
    N_("Edit project properties"), G_CALLBACK (openldev_project_preferences) },
  { "ProjectInfo", "PROJECTINFO", N_("_Project Info"), NULL, 
    N_("View project info"), G_CALLBACK (openldev_project_info) },
  
  { "BuildCompile", "COMPILE_SMALL", N_("_Compile"), "F9", 
    N_("Compile current file"), G_CALLBACK (openldev_build_compile) },
  { "BuildCompileToolbar", "COMPILE", N_("_Compile"), NULL, 
    N_("Compile current file"), G_CALLBACK (openldev_build_compile) },
  { "BuildBuild", "BUILD_SMALL", N_("_Build"), "F11", 
    N_("Build project tree"), G_CALLBACK (openldev_build_project) },
  { "BuildBuildToolbar", "BUILD", N_("_Build"), NULL, 
    N_("Build project tree"), G_CALLBACK (openldev_build_project) },
  { "BuildExecute", GTK_STOCK_EXECUTE, N_("_Execute"), "F3", 
    N_("Execute project"), G_CALLBACK (openldev_build_execute) },
  { "BuildAutogen", GTK_STOCK_CONVERT, N_("_Auto Generate"), NULL, 
    N_("Auto generate and configure"), G_CALLBACK (openldev_build_auto_generate) },
  { "BuildConfigure", NULL, N_("Con_figure"), NULL, 
    N_("Run configure script"), G_CALLBACK (openldev_build_configure) },
  { "BuildClean", GTK_STOCK_CLEAR, N_("_Make Clean"), NULL, 
    N_("Clean distribution tree"), G_CALLBACK (openldev_build_clean) },
  { "BuildStop", GTK_STOCK_STOP, N_("_Stop"), "F12", 
    N_("Stop current action"), G_CALLBACK (openldev_build_stop) },
  
  { "HelpDocs", GTK_STOCK_HELP, N_("_Documentation"), "F1", 
    N_("View project documentation"), G_CALLBACK (openldev_help_documentation) },
  { "HelpBugs", NULL, N_("_Report Bugs"), NULL, 
    N_("Report bugs"), G_CALLBACK (openldev_help_report_bug) },
  { "HelpAbout", GTK_STOCK_ABOUT, N_("_About"), "<alt>a",
    N_("About OpenLDev"), G_CALLBACK (openldev_help_about) },
  
  { "SearchOutputGo", GTK_STOCK_JUMP_TO, N_("Go To Match"), NULL, 
    N_("Go to line of match"), G_CALLBACK (openldev_jump_to_search_output) },
  { "SearchOutputCopy", GTK_STOCK_COPY, N_("Copy"), NULL, 
    N_("Copy search text"), G_CALLBACK (openldev_copy_output) },
  { "SearchOutputClear", GTK_STOCK_CLEAR, N_("Clear"), NULL, 
    N_("Clear serach output"), G_CALLBACK (openldev_clear_search_output) },
  
  { "BuildOutputGo", GTK_STOCK_JUMP_TO, N_("Go To Error"), NULL, 
    N_("Go to line of error"), G_CALLBACK (openldev_jump_to_build_output) },
  { "BuildOutputCopy", GTK_STOCK_COPY, N_("Copy"), NULL, 
    N_("Copy error text"), G_CALLBACK (openldev_copy_output) },
  { "BuildOutputClear", GTK_STOCK_CLEAR, N_("Clear"), NULL, 
    N_("Clear build output"), G_CALLBACK (openldev_clear_build_output) },
  
  { "OpenFileBrowser", GTK_STOCK_OPEN, N_("_Open File"), NULL, 
    N_("Open an existing file"), G_CALLBACK (openldev_open_file_browser) },
  { "RefreshFileBrowser", GTK_STOCK_REFRESH, N_("Refresh"), NULL, 
    N_("Refresh file browser"), G_CALLBACK (openldev_refresh_file_browser) }
};

#define NUM_ENTRIES_TOGGLE 3
static GtkToggleActionEntry toggle_entries[] = {
  { "ViewToolbar", NULL, N_("_Toolbar"), NULL, 
    N_("Show or hide toolbar"), G_CALLBACK (openldev_view_toolbar), TRUE },
  { "ViewStatusBar", NULL, N_("_Status Bar"), NULL, 
    N_("Show or hide status bar"), G_CALLBACK (openldev_view_statusbar), TRUE },
  { "ViewLineNumbering", NULL, N_("_Line Numbering"), NULL, 
    N_("Show or hide line numbers"), G_CALLBACK (openldev_view_line_numbering), TRUE }
};

#endif
